model Queue 
  flow Real r "In-flow";
  discrete Boolean queue_full(start=false) "Queue full";
  
  parameter Real B=1 "Bandwidth";
  parameter Real Qmax=1 "Max queue size";
  
protected 
  discrete Integer status(start=0) "Queue status";
  // 0 empty, 2 full, 1 otherwise
  Real q(start=0) "Queue size";
  

equation 
  // no longer empty || no longer full || empty || full || no change
  status = if pre(status) == 0 and r > B then 1 else if pre(status) == 2 and r
     < B then 1 else if pre(status) == 1 and q <= 0 and r < B then 0 else if 
    pre(status) == 1 and q >= Qmax and r > B then 2 else pre(status);
  queue_full = pre(status) == 1 and status == 2;
  der(q) = if status == 1 then r - B else 0;
end Queue;


model Controller 
  flow Real r(start=0) "out-flow";
  discrete Boolean queue_full "Queue full";
  
  parameter Real a=.1 "Additive constant";
  parameter Real m=.5 "Multiplicative constant";
  

equation 
  der(r) = a;
  when pre(queue_full) then
    reinit(r, r*m);
  end when;
  
end Controller;


model System 
  Queue queue;
  Controller contr;

equation 
  queue.r = contr.r;
  queue.queue_full = contr.queue_full;
end System;

