connector DataInterface 
  flow Real r "flow rate";
end DataInterface;


connector QueueFullInterface 
  Boolean event(start=false) "flow rate";
  annotation (Coordsys(
      extent=[-100, -100; 100, 100], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.4, 
      y=0.4, 
      width=0.6, 
      height=0.6));
end QueueFullInterface;


model Queue 
  parameter Real B=1 "Bandwidth";
  parameter Real Qmax=1 "Max queue size";
  
protected 
  Real q(start=0) "queue size";
  
  // 0 empty, 2 full, 1 otherwise
  discrete Integer status(start=0) "queue status";
public 
  DataInterface in_flow "in-flow" annotation (extent=[-20, 80; 40, 100]);
  QueueFullInterface queue_full "Queue full" annotation (extent=[-40, -92; 60, 
        -72]);
  annotation (
    Coordsys(
      extent=[-100, -100; 100, 100], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.07, 
      y=0.01, 
      width=0.86, 
      height=0.95), 
    Diagram, 
    Icon(
      Line(points=[-20, 60; -20, -60; 40, -60; 40, 60]), 
      Line(points=[-20, -40; 40, -40]), 
      Line(points=[-20, -20; 40, -20]), 
      Line(points=[-20, 0; -14, 0; 40, 0]), 
      Line(points=[-20, 20; 40, 20])));
equation 
  // input flow port (in flow, thus positive)
  der(q) = if status == 1 then in_flow.r - B else 0;
  
  // no longer empty || no longer full || empty || full || same status
  status = if pre(status) == 0 and in_flow.r > B then 1 else if pre(status) == 
    2 and in_flow.r < B then 1 else if pre(status) == 1 and q <= 0 and in_flow.
    r < B then 0 else if pre(status) == 1 and q >= Qmax and in_flow.r > B then 
    2 else pre(status);
  
  queue_full.event = if pre(status) == 1 and status == 2 then true else false;
  
end Queue;


model Controller 
  parameter Real a=.1 "Additive constant";
  parameter Real m=.5 "Multiplicative constant";
  QueueFullInterface queue_full "Queue full" annotation (extent=[-98, -100; 98
        , -70]);
  DataInterface out_flow "out-flow" annotation (extent=[64, -16; 190, 16]);
  annotation (
    Coordsys(
      extent=[-100, -100; 100, 100], 
      grid=[2, 2], 
      component=[20, 20]), 
    Window(
      x=0.37, 
      y=0.01, 
      width=0.6, 
      height=0.6), 
    Icon(Ellipse(extent=[-62, 60; 60, -60], style(fillColor=74)), Polygon(
          points=[-40, 44; -40, -48; 60, -2; -40, 44], style(fillColor=7))));
equation 
  
  // outflow, thus negative
  der(out_flow.r) = -a;
  
  when pre(queue_full.event) then
    reinit(out_flow.r, out_flow.r*m);
  end when;
  
end Controller;


model System 
  Queue queue annotation (extent=[-19, -43; 50, 40]);
  Controller contr1 annotation (extent=[-89, -39; -54, -6]);
  annotation (Coordsys(
      extent=[-100, -100; 100, 100], 
      grid=[2, 2], 
      component=[20, 20]), Window(
      x=0.29, 
      y=0.07, 
      width=0.68, 
      height=0.9));
  Controller contr2 annotation (extent=[-88, 36; -54, 2]);
equation 
  connect(queue.in_flow, contr1.out_flow) annotation (points=[18.95, 
        33.85; 18.95, 38.85; -45.25, -22.83; -45.25, -22.83]);
  connect(contr2.out_flow, queue.in_flow) annotation (points=[-47.1, 17.34; -39
        , 17.34; 18.95, 38.85; 18.95, 33.85]);
  connect(queue.queue_full, contr1.queue_full) annotation (points=[18.95, 
        -35.87; 18.95, -42.35; -71.5, -42.35; -71.5, -37.35]);
  connect(queue.queue_full, contr2.queue_full) annotation (points=[24.95, 
        -33.87; 24.95, -38.87; 26, -44; 36, -44; 36, 46; -65, 46; -65, 34.3]);
end System;

